/************************************************************************/
/*                                                                      */
/*  PmodJSTK2CalibrationDemo.ino  -- Example Sketch for PmodJSTK2       */
/*                                                                      */
/************************************************************************/
/*  Author: James Colvin                                                */
/*  Copyright (c) 2016, Digilent                                        */
/************************************************************************/
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
*/
/************************************************************************/
/*  Module Description:                                                 */
/*                                                                      */
/*  This sketch meaures all of the currently loaded calibration values  */
/*  and prints them to the Serial Monitor. The calibration sequence is  */
/*  then started where the user has 10 seconds to move the joystick     */
/*  around in order to re-calibrate the module.                         */
/*                                                                      */
/*  After calibration is complete
/*                                                                      */
/************************************************************************/
/*  Revision History:                                                   */
/*                                                                      */
/*  6/02/2016(James C): Created                                         */
/*                                                                      */
/************************************************************************/
/************************************************************************/
/*  Board Support:                                                      */
/*                                                                      */
/*  chipKIT WF32:               DSPI0 with SS as pin 10 (not pin 44)    */
/*                       MOSI - 11, MISO - 12, and SCLK - 13 as normal  */
/*  chipKIT Uno32, uC32, WiFIRE  DSPI0 (pins 10, 11, 12, 13 are default)*/
/*                                                                      */
/************************************************************************/

//including the required libraries
#include <DSPI.h>
#include <PmodJSTK2.h>
//creating an instance of the JSTK2 class
JSTK2 jstk2;

void setup(){
  //start up the Serial Monitor
  Serial.begin(9600);
  
  //intialize the JSTK2 class, DPSI0 (by default),
  //and have the library incorporate necessary delays
  //for you (by default)
  jstk2.begin();
  
  //reload calibration values from flash in case users run this sketch
  //multiple times without power-cycling the system board in order to 
  //make sure that the original collected values are correct and not 
  //the RAM values from the previous run-through.
  Serial.println();Serial.println("Starting up the sketch with a reloading of");
  Serial.println("calibration values from Flash memory");Serial.println();
  jstk2.rldFromFlash();
  
  //collect all of the calibration constants (loaded from flash upon power-up)
  //and print them out
  jstk2.getAllCalibrationConstants();
  Serial.print("power-up smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("power-up smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("power-up smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("power-up smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("power-up smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("power-up smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("power-up smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("power-up smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  //Store all of these collected calibration values to compare against later
  int XMin = jstk2.jstkCal.smpXMin;
  int XMax = jstk2.jstkCal.smpXMax;
  int YMin = jstk2.jstkCal.smpYMin;
  int YMax = jstk2.jstkCal.smpYMax;
  int XCenterMin = jstk2.jstkCal.smpXCenterMin;
  int XCenterMax = jstk2.jstkCal.smpXCenterMax;
  int YCenterMin = jstk2.jstkCal.smpYCenterMin;
  int YCenterMax = jstk2.jstkCal.smpYCenterMax;
  
  Serial.println();
  Serial.println("calibrating, rotate the Joystick around in all directions");
  /*
  If the joystick has been rotated around and then detected to be at rest
  for a full second within 10 seconds time, the blue LED will stop flashing
  and the green LED will flash twice indicating that calibration was succesful.
  If neither of those conditions are met within 10 seconds, the blue LED will
  stop flashing, and the red LED will flash twice indicating that the 
  calibration sequence was unsuccessful.
  */
  jstk2.calibrate();
  jstk2.calibrationDelay();
  Serial.println("calibration complete");Serial.println();
  
  //re-collect all of the calibration constants and print them out
  jstk2.getAllCalibrationConstants();
  Serial.print("new smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("new smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("new smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("new smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("new smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("new smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("new smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("new smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  
  //reload all of the original calibration values from the PmodJSTK2's flash memory
  Serial.println();Serial.println("reload calibration values from flash");
  jstk2.rldFromFlash();
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  Serial.println();
  Serial.print("flash smpXMin: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("flash smpXMax: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("flash smpYMin: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("flash smpYMax: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("flash smpXCenterMin: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("flash smpXCenterMax: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("flash smpYCenterMin: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("flash smpYCenterMax: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  
  //Now we will load specific calibration values (10-bit) into RAM
  /*
  Note that provided parameters must meet certain criteria in order to be 
  //successfully loaded into the RAM of the embedded PIC16
  //For example, a provided 10-bit calibration value for the minimum Y 
  //axis value must be between an internally set Y minimum value and an 
  //internally set Y Center minimum value in order to be successfully set.
  */
  Serial.println();
  jstk2.setCalXMin(XMin+5);
  jstk2.setCalXMax(XMax-5);
  jstk2.setCalYMin(YMin+5);
  jstk2.setCalYMax(YMax-5);
  jstk2.setCalXCenMin(XCenterMin+5);
  jstk2.setCalXCenMax(XCenterMax-5);
  jstk2.setCalYCenMin(YCenterMin+5);
  jstk2.setCalYCenMax(YCenterMax-5);
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  Serial.print("smpXMin+5: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("smpXMax-5: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("smpYMin+5: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("smpYMax-5: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("smpXCenterMin+5: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("smpXCenterMax-5: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("smpYCenterMin+5: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("smpYCenterMax-5: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  //Now we will load specific calibration values (10-bit) into RAM
  //but this time with alternate functions, just for fun.
  /*
  Note that provided parameters must meet certain criteria in order to be 
  //successfully loaded into the RAM of the embedded PIC16
  //For example, a provided 10-bit calibration value for the minimum Y 
  //axis value must be between an internally set Y minimum value and an 
  //internally set Y Center minimum value in order to be successfully set.
  */
  Serial.println();
  jstk2.setCalXMinMax(XMin+10, XMax-10);
  jstk2.setCalYMinMax(YMin+10, YMax-10);
  jstk2.setCalXCenMinMax(XCenterMin+10, XCenterMax-10);
  jstk2.setCalYCenMinMax(YCenterMin+10, YCenterMax-10);
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  Serial.print("new smpXMin+10: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("new smpXMax-10: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("new smpYMin+10: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("new smpYMax-10: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("new smpXCenterMin+10: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("new smpXCenterMax-10: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("new smpYCenterMin+10: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("new smpYCenterMax-10: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  //Now we will write these values to flash memory and then re-write the original
  //calibration values to flash memory
  Serial.println();Serial.println("Writing the current calibration values in RAM to the embedded Flash memory");
  jstk2.writeFlash();
  
  Serial.println();Serial.println("confirm values written into flash are accurate");
  jstk2.rldFromFlash();
  
  //re-collect all of the calibration constants and print them out
  jstk2.getAllCalibrationConstants();
  
  Serial.print("flash smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("flash smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("flash smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("flash smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("flash smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("flash smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("flash smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("flash smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  
  Serial.println();Serial.println("Writing other values to RAM");
  jstk2.setCalXMin(XMin+5);
  jstk2.setCalXMax(XMax-5);
  jstk2.setCalYMin(YMin+5);
  jstk2.setCalYMax(YMax-5);
  jstk2.setCalXCenMin(XCenterMin+5);
  jstk2.setCalXCenMax(XCenterMax-5);
  jstk2.setCalYCenMin(YCenterMin+5);
  jstk2.setCalYCenMax(YCenterMax-5);
  
  Serial.println();Serial.println("measuring loaded calibration constants and comparing them to the original, not what's currently in flash");
  Serial.println("this is to confirm that it worked");
  jstk2.getAllCalibrationConstants();
  
  Serial.print("smpXMin+5: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("smpXMax-5: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("smpYMin+5: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("smpYMax-5: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("smpXCenterMin+5: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("smpXCenterMax-5: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("smpYCenterMin+5: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("smpYCenterMax-5: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  //reload all of the calibration values from the PmodJSTK2's flash memory
  Serial.println();Serial.println("reload calibration values from flash");
  jstk2.rldFromFlash();
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  
  Serial.println();
  Serial.print("flash smpXMin: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("flash smpXMax: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("flash smpYMin: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("flash smpYMax: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("flash smpXCenterMin: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("flash smpXCenterMax: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("flash smpYCenterMin: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("flash smpYCenterMax: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  Serial.println();Serial.println("Load the factory flash values back into RAM, and then back into flash");
  jstk2.setCalXMinMax(XMin, XMax);
  jstk2.setCalYMinMax(YMin, YMax);
  jstk2.setCalXCenMinMax(XCenterMin, XCenterMax);
  jstk2.setCalYCenMinMax(YCenterMin, YCenterMax);
  
  jstk2.writeFlash();
  
  Serial.println();Serial.println("confirm we're all back to normal");
  //re-collect all of the calibration constants and confirm it's now correct
  jstk2.getAllCalibrationConstants();
  Serial.print("new smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("new smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("new smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("new smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("new smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("new smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("new smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("new smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
}

void loop(){
  //do nothing
}
