/************************************************************************/
/*                                                                      */
/*  PmodJSTK2_LedBtnsDemo.ino -- Example Sketch for PmodJSTK2           */
/*                                                                      */
/************************************************************************/
/*  Author: James Colvin                                                */
/*  Copyright (c) 2016, Digilent                                        */
/************************************************************************/
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
*/
/************************************************************************/
/*  Module Description:                                                 */
/*                                                                      */
/*  This sketch takes the state of the two buttons and the cuurent      */
/*  axis values and correspondingly adjusts the color of the RGB LED.   */
/*                                                                      */
/************************************************************************/
/*  Revision History:                                                   */
/*                                                                      */
/*  6/01/2016(James C): Created                                         */
/*                                                                      */
/************************************************************************/
/************************************************************************/
/*  Board Support:                                                      */
/*                                                                      */
/*  chipKIT WF32:               DSPI0 with SS as pin 10 (not pin 44)    */
/*                       MOSI - 11, MISO - 12, and SCLK - 13 as normal  */
/*  chipKIT Uno32, uC32, WiFIRE  DSPI0 (pins 10, 11, 12, 13 are default)*/
/*                                                                      */
/************************************************************************/

//including the required libraries
#include <DSPI.h>
#include <PmodJSTK2.h>
//creating an instance of the JSTK2 class
JSTK2 jstk2;

void setup(){
  //intialize the JSTK2 class, DPSI0 (by default),
  //and have the library incorporate necessary delays
  //for you (by default)
  jstk2.begin();
}

void loop(){
  //get the 8-bit position variables of the joystick
  jstk2.getPosition();
  //set the colors on the RGB LED based on the two
  //JSTK2 axis values from the getPosition function
  //and the blue LED based on the state of the two
  //buttons
  
  //the red and green LEDs have a full range of brightness
  //from the 8-bit position values but there are 4 possible 
  //states for the blue LED from the two buttons
  //  no buttons pressed             - 0% brightness
  //  joystick shaft button pressed  - 33% brightness
  //  trigger button pressed         - 67% brightness
  //  both buttons pressed           - 100% brightness
  if((jstk2.jstkPkt.fsBtns & 0x03) == 0){
    jstk2.setLedRGB(jstk2.jstkPkt.posX, jstk2.jstkPkt.posY, 255*0/100);
  }
  else if((jstk2.jstkPkt.fsBtns & 0x03) == 1){
    jstk2.setLedRGB(jstk2.jstkPkt.posX, jstk2.jstkPkt.posY, 255*33/100);
  }
  else if((jstk2.jstkPkt.fsBtns & 0x03) == 2){
    jstk2.setLedRGB(jstk2.jstkPkt.posX, jstk2.jstkPkt.posY, 255*67/100);
  }
  else if((jstk2.jstkPkt.fsBtns & 0x03) == 3){
    jstk2.setLedRGB(jstk2.jstkPkt.posX, jstk2.jstkPkt.posY, 255*100/100);
  }
}
